#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <stdint.h>
#include <ctype.h>

#define FILE_NAME "/mnt/huge/page"

#ifdef MEMLOCK_IF_AVAILABLE
#define MMAP_FLAGS  (MAP_SHARED|MAP_LOCKED)
#else
#define MMAP_FLAGS  (MAP_SHARED)
#endif

static int fd;
static int size_mapped;

void* alloc_func(int size)
{
  void* ptr;

  fd = open( FILE_NAME, O_CREAT | O_RDWR, 0755);
  if (fd < 0) {
    printf("Error: hugetlbfs is not ready\n");
    exit(1);
  }

  size_mapped = size;
  ptr = mmap(0, size, PROT_READ|PROT_WRITE,  MMAP_FLAGS,fd, 0);
  if (ptr == MAP_FAILED) {
    perror("mmap");
    unlink(FILE_NAME);
    exit(1);
  }

  return ptr;

}

void free_func(void* ptr)
{
  munmap(ptr,size_mapped);
  close(fd);
  unlink(FILE_NAME);
}

void   bench_iteration_report( void* (allocate_func)(int size), 
			       void  (free_func)(void* ptr) );


int  get_hugepage_size();
unsigned long long get_cpufreq_HZ();


#define   PRINT_MMAP_FLAGS(flags,name,cnt)	\
  do {						\
    if((flags)&(name)) {			\
      if(cnt>0) printf("|");			\
      printf("%s",#name);			\
      cnt++;					\
    }						\
  } while(0);


static void print_mmap_flags(int flags)
{
  int cnt=0;

  PRINT_MMAP_FLAGS(flags,MAP_SHARED ,cnt);
  PRINT_MMAP_FLAGS(flags,MAP_PRIVATE,cnt);
  PRINT_MMAP_FLAGS(flags,MAP_LOCKED,cnt);
  PRINT_MMAP_FLAGS(flags,MAP_ANONYMOUS,cnt);
}

int main(int argc, char* argv[])
{
  printf("# PAGE_SIZE=%d\n", get_hugepage_size() );
  printf("# TICKS_PER_SEC=%llu\n",     get_cpufreq_HZ() );

  printf("# MMAP_FLAGS=");
  print_mmap_flags(MMAP_FLAGS); puts("");

  bench_iteration_report( alloc_func, free_func );

  return 0;
}



