/*
  The license of this code is LGPL. See license.LGPL
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "membench_kernel.h"

#define   MEMLATENCY_PROGRAM_VERSION  "1.0"


/* benchmark parameters */
static int n_try = 5;

static int bench_size = 64*1024*1024;

/*
  Default is 64MB 

  You can change allocation size with MEMLATENCY_BENCH_BUFSIZE
  environment variable
*/

static int iter_set[] = { 1000,
			  10000,
			  100000,
			  1000000,
			  10000000 };

//			  100000000 };



void   bench_iteration_report( void* (allocate_func)(int size), 
			       void  (free_func)(void* ptr)
			       )
{
  void *ptr;
  int  i,idx;
  tick_t ret;
  char *str;

  str = getenv("MEMLATENCY_BENCH_BUFSIZE");
  if(str) {
    int  bufsize = atoi(str);
    bench_size = bufsize;   
  }

  printf("# BUFSIZE=%d\n", bench_size);
#ifdef DATATYPE_NAME
  printf("# DATATYPE=%s\n", datatypeid_name());
#endif
#ifdef EXTRAFLOPS
  printf("# EXTRAFLOPS\n");
#endif
  printf("# VERSION=%s\n", MEMLATENCY_PROGRAM_VERSION);
  
  printf("#\n");
  printf("# iteration  randomread  randomcopy  steamcopy\n");

  for(idx=0;idx<sizeof(iter_set)/sizeof(int);idx++) {
    for(i=0; i<n_try; i++) {

      printf("%12d ", iter_set[idx]);

      ptr = allocate_func(bench_size);
      ret=method_randomread(ptr,bench_size,iter_set[idx]);
      free_func(ptr);
      printf("%10llu ", TICK_UINT64(ret));

      ptr = allocate_func(bench_size);
      ret=method_randomcopy(ptr,bench_size,iter_set[idx]);
      free_func(ptr);
      printf("%12llu ", TICK_UINT64(ret));

      ptr = allocate_func(bench_size);
      ret=method_streamcopy(ptr,bench_size,iter_set[idx]);
      free_func(ptr);
      printf("%12llu ", TICK_UINT64(ret));

      puts("");

    }
  }
}
