#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mman.h>

#ifdef MEMLOCK_IF_AVAILABLE
#define MEM_LOCKED  (1)
#endif

static int alloced_size=0;

void* alloc_func(int size)
{
  void* ptr;
  ptr = malloc(size);
#ifdef MEM_LOCKED
  mlock(ptr,size); 
#endif
  alloced_size=size;

  if( !ptr ) {
    perror("malloc");
    exit(1);
  }
  return ptr;
}

void free_func(void* ptr)
{
#ifdef MEM_LOCKED
  munlock(ptr,alloced_size); 
#endif

  free(ptr);
}

void   bench_iteration_report( void* (allocate_func)(int size), 
			       void  (free_func)(void* ptr) );

unsigned long long get_cpufreq_HZ();

int main(int argc, char* argv[])
{
  printf("# PAGE_SIZE=%d\n", getpagesize() );
  printf("# TICKS_PER_SEC=%llu\n",     get_cpufreq_HZ() );
#ifdef MEM_LOCKED
  printf("# MLOCKED\n");
#endif



  bench_iteration_report( alloc_func, free_func );

  return 0;
}



