#!/bin/bash

#  The license of this code is LGPL. See license.LGPL


if [ -z "$1" ] ; then
    echo "Usage: $0 [filename]"
    exit 
fi

bn=`echo $1|sed s/-huge.dat//|sed s/-regular.dat//`


if [ ! -f "$bn-huge.dat" ] ; then
    echo "$bn-huge.dat is missing"
    exit
fi

if [ ! -f  "$bn-regular.dat" ] ; then
    echo "$bn-regular.dat is missing"
    exit
fi


ANALYZECMD=./analyze-data.pl

if [ ! -f $ANALYZECMD ] ; then
    echo "$ANALYZECMD not found"
    exit 1
fi


TMP_HUGE=tmp-$bn-huge-gr.dat
TMP_REG=tmp-$bn-regular-gr.dat
$ANALYZECMD < $bn-huge.dat    > $TMP_HUGE
$ANALYZECMD < $bn-regular.dat > $TMP_REG



cat - << EOL > a.gp
set terminal png
set output "$bn-randomread-compare.png"
set title "randomread"
set xlabel "Number of iteration"
set ylabel "Latency(nsec) per iteration"
set yrange [0:]
set logscale x
pl  '$TMP_HUGE' using 1:2  w l t "2M page", \
    '$TMP_HUGE' using 1:2:3 w yerrorbars  t "", \
    '$TMP_REG' using 1:2 w l t "4K page" , \
    '$TMP_REG' using 1:2:3 w yerrorbars t ""

repl
EOL

gnuplot a.gp


cat - << EOL > a.gp
set terminal png
set output "$bn-randomcopy-compare.png"
set title "randomcopy"
set xlabel "Number of iteration"
set ylabel "Latency(nsec) per iteration"
set logscale x
set yrange [0:]
pl  '$TMP_HUGE' using 1:4  w l t "2M page", \
    '$TMP_HUGE' using 1:4:5 w yerrorbars  t "", \
    '$TMP_REG' using 1:4 w l t "4K page" , \
    '$TMP_REG' using 1:4:5 w yerrorbars t ""

repl
EOL

gnuplot a.gp


cat - << EOL > a.gp
set terminal png
set output "$bn-streamcopy-compare.png"
set title "streamcopy"
set xlabel "Number of iteration"
set ylabel "Latency(nsec) per iteration"
set logscale x
set yrange [0:]
pl  '$TMP_HUGE' using 1:6  w l t "2M page", \
    '$TMP_HUGE' using 1:6:7 w yerrorbars  t "", \
    '$TMP_REG' using 1:6 w l t "4K page" , \
    '$TMP_REG' using 1:6:7 w yerrorbars t ""

repl
EOL

gnuplot a.gp

rm -f $TMP_HUGE $TMP_REG
rm -f a.gp
