#!/bin/bash

#  The license of this code is LGPL. See license.LGPL

ANALYZECMD=./analyze-data.pl

if [ -z "$1" ] ; then
    echo "Usage: $0 basename"
    exit 
fi


bn=`echo $1|sed s/-huge.dat//|sed s/-regular.dat//`


if [ ! -f "$bn-huge.dat" ] ; then
    echo "$bn-huge.dat is missing"
    exit
fi

if [ ! -f  "$bn-regular.dat" ] ; then
    echo "$bn-regular.dat is missing"
    exit
fi



if [ ! -f $ANALYZECMD ] ; then
    echo "$ANALYZECMD not found"
    exit 1
fi


TMP_HUGE=tmp-$bn-statictlb-gr.dat
TMP_REG=tmp-$bn-regular-gr.dat
$ANALYZECMD  < $bn-statictlb.dat  > $TMP_HUGE
$ANALYZECMD  < $bn-regular.dat    > $TMP_REG


cat - << EOL > a.gp
set terminal png
set output "$bn-randomread-compare.png"
set title "randomread"
set xlabel "Number of iteration"
set ylabel "Latency(nsec) per iteration"
set yrange [0:]
set logscale x
pl  '$TMP_HUGE' using 1:2   t "Static TLB" w l, \
    '$TMP_HUGE' using 1:2:3 t "" w yerrorbars  , \
    '$TMP_REG'  using 1:2   t "Dynamic TLB(4KB)" w l , \
    '$TMP_REG'  using 1:2:3 t "" w yerrorbars
repl
EOL



gnuplot a.gp

cat - << EOL > a.gp
set terminal png
set output "$bn-randomcopy-compare.png"
set title "randomcopy"
set xlabel "Number of iteration"
set ylabel "Latency(nsec) per iteration"
set logscale x
set yrange [0:]
pl  '$TMP_HUGE' using 1:4   t "Static TLB" w l, \
    '$TMP_HUGE' using 1:4:5 t "" w yerrorbars  , \
    '$TMP_REG'  using 1:4   t "Dyamic TLB(4KB)"  w l, \
    '$TMP_REG'  using 1:4:5 t "" w yerrorbars 
repl
EOL

gnuplot a.gp



cat - << EOL > a.gp
set terminal png
set output "$bn-streamcopy-compare.png"
set title "streamcopy"
set xlabel "Number of iteration"
set ylabel "Latency(nsec) per iteration"
set logscale x
set yrange [0:]
pl  '$TMP_HUGE' using 1:6   t "Static TLB" w l, \
    '$TMP_HUGE' using 1:6:7 t "" w yerrorbars , \
    '$TMP_REG'  using 1:6   t "Dynamic TLB(4KB)" w l , \
    '$TMP_REG'  using 1:6:7 t "" w yerrorbars 

repl
EOL

gnuplot a.gp

rm -f $TMP_HUGE $TMP_REG
rm -f a.gp

