/*
  The license of this code is LGPL. See license.LGPL
*/

/*
  NOTE: 
  
  This code should be easy to port to early Linux kernel stage with
  minimum modification because we want to measure the memory
  performance on no-paging environment.
*/

#include "membench_kernel.h"

#ifdef DATATYPE_DOUBLE
typedef double  data_t;
#else
typedef int  data_t;
#endif


#ifdef DATATYPE_NAME
static int is_floatingpoint()
{
  data_t  d = (data_t)1;
  char*   p = (char*)&d;
  int     i,tmp;

  tmp = 0;
  for(i=0;i<sizeof(d);i++) {
    tmp += (int)p[i];
  }

  if( tmp!=1 ) return 1;

  return 0;
}

static char datatypeid_name_buf[16];

#include <stdio.h>

const char* datatypeid_name()
{
  snprintf(datatypeid_name_buf,sizeof(datatypeid_name_buf),
	   "%s%d",
	   (is_floatingpoint()?"FP":"INT"),
	   sizeof(data_t));
  return datatypeid_name_buf;
}
#endif


/* from twister.c */
typedef unsigned long uint32;
void    seedMT(uint32 seed);
uint32  randomMT(void);


#if defined(__i386__) || defined(__x86_64__)


static tick_t gettick(void)
{
  tick_t t;

#ifdef HAS_LONGLONG
  __asm__ __volatile__("rdtsc" : "=A" (t.val));
#else
  __asm__ __volatile__("rdtsc" : "=a" (t.low), "=d" (t.high));
#endif

  return t;
}

#elif defined(__powerpc__)
static tick_t gettick(void)
{
  tick_t t;
  unsigned long upper, lower,tmp;

  __asm__ __volatile__ (
                "1:                     \n"
                "\tmftbu   %0           \n"
                "\tmftb    %1           \n"
                "\tmftbu   %2           \n"
                "\tcmpw    %2,%0        \n"
                "\tbne     1b           \n"
                : "=r"(upper),"=r"(lower),"=r"(tmp)
                );
#ifdef HAS_LONGLONG
  t.val  = ((unsigned long long)upper<<32)|lower;
#else
  t.low  = lower;
  t.high = upper;
#endif

  return t;
}
#endif




#ifdef HAS_LONGLONG
static tick_t difftick(tick_t a, tick_t b )
{
  tick_t ret;
  ret.val = a.val - b.val;
  return ret;
}
#else
static tick_t difftick(tick_t a, tick_t b )
{
  tick_t c;

  if( a.low >= b.low ) {
    c.low = a.low - b.low;
  } else {
    a.high --;
    c.low = -1;
    c.low -= (b.low - a.low);
  }

  c.high = a.high - b.high;  
  /* don't care the case for a.high < b.high */

  return c;
}
#endif



/*
  return a tick_t value that the benchmark loop spent.
*/
tick_t method_randomread( void* ptr, unsigned size, int niter )
{
  data_t    *array = (data_t*)ptr;
  int       size_array = size/sizeof(data_t);
  tick_t tv_start, tv_stop;
  volatile int next_index=0;  /* use volatile to avoid optimization effect */
  int i;

  for( i=0; i<size_array; i++ )  array[i]=randomMT()%size_array;
  next_index=randomMT()%size_array;

  tv_start = gettick();
  for( i=0; i<niter ; i++ )  {
#ifdef EXTRAFLOPS
    {
      double a;
      a = (double)( ( (int)array[next_index]%10)+1 );
      next_index = (int)( (double)array[next_index]*a + a );
      next_index = next_index % size_array;
      a = (double)( ( (int)array[next_index]%10)+1 );
      next_index = (int)( (double)array[next_index]*a + a );
      next_index = next_index % size_array;
   }
#else
    next_index = (int)array[next_index];
    next_index = (int)array[next_index];
#endif

  }
  tv_stop  = gettick();
  
  return difftick( tv_stop, tv_start );
}


tick_t method_randomcopy( void* ptr, unsigned size, int niter )
{
  data_t     *array = (data_t*)ptr;
  int     size_array = size/sizeof(data_t);
  tick_t  tv_start, tv_stop;
  tick_t  rnd_oht;
  int     p1,p2;
  int     i;

  tv_start = gettick();
  for( i=0; i<niter; i++ ) {
    randomMT();
    randomMT();
  }
  tv_stop  = gettick();
  rnd_oht = difftick(tv_stop,tv_start);

  tv_start = gettick();
  for( i=0; i<niter; i++ ) {
    p1=(randomMT() % size_array);
    p2=(randomMT() % size_array);
#ifdef EXTRAFLOPS
    // extra 2 flops
    array[p1] = (double)array[p2] * (double) p1 + (double)p2;
#else
    array[p1] = array[p2];
#endif

  }
  tv_stop  = gettick();

  return difftick( difftick( tv_stop, tv_start), rnd_oht  );
}


tick_t method_streamcopy(void* ptr, unsigned size, int niter)
{
  tick_t tv_start, tv_stop;
  char   *cptr = ptr;
  data_t *src  = (data_t*)(cptr+0);
  data_t *dst  = (data_t*)(cptr+size/2);
  int size_src = (size/2)/sizeof(data_t);
  int i;
  int pos;

  tv_start = gettick();
  for(i=0;i<niter;i++) {
#ifdef EXTRAFLOPS
    // extra 2 flops
    pos = i%size_src;
    dst[pos]=(data_t) ((double)src[pos] * (double)i + (double)i );
#else
    pos = i%size_src;
    dst[pos]=src[pos];
#endif

  }
  tv_stop = gettick();

  return  difftick(tv_stop,tv_start);
}

